@@ -4,6 +4,8 @@ use 5.006;
 use strict;
 use Script::Toolbox::Util::Opt;
 use Script::Toolbox::Util::Formatter;
+use Script::Toolbox::Util::Menues;
+use Script::Toolbox::TableO;
 use IO::File;
 use IO::Dir;
 use Data::Dumper;
@@ -30,7 +32,7 @@ our @EXPORT = qw(
 	
 );
 
-our $VERSION = '0.49';
+#our $VERSION = '0.50';
 
 # Preloaded methods go here.
 sub _getKV(@);
@@ -215,19 +217,20 @@ use Fatal qw(IO::File::open);
 
 
 #------------------------------------------------------------------------------
+# Format $param as table and return a reference to the result array.
 #------------------------------------------------------------------------------
 sub Table(@)
 {
 	my ($self, $param, $separator) = @_;
+    if( ref $self ne 'Script::Toolbox') { # only for compatibility
+        $separator = $param;              # to versions befor 0.50
+        $param     = $self;
+        $self      = undef;
+    }
 
-	return undef	if( _noData( $param ) );
-	my $para  = $self->_normParam($param, $separator);
-
-	my $form  = Script::Toolbox::Util::Formatter->new( $para );
-	my $result= $form->matrix();
-	return $result	if( ref $param eq 'ARRAY' || !defined $param->{'sumCols'} );
-
-	return $form->sumBy($result, $param->{'sumCols'}, $param->{'notGroupBy'});
+    my $t = Script::Toolbox::TableO->new($param, $separator);
+    my @T =  $t->asArray();
+    return \@T;
 }
 
 #------------------------------------------------------------------------------
@@ -264,7 +267,7 @@ sub _noData($)
 #------------------------------------------------------------------------------
 sub _normParam($$)
 {
-	my ($self, $param, $separator) = @_;
+	my ($param, $separator) = @_;
 
 	if( ref $param eq 'HASH' )
 	{
@@ -830,21 +833,17 @@ sub Now(@)
 #------------------------------------------------------------------------------
 #------------------------------------------------------------------------------
 sub _printMenueHeader($) {
-     my ($opts) = @_;
-     foreach my $op ( @{$opts} ){
-         next     if( ! $op->{'header'} );
-         printf "%s\n", $op->{'header'};
-     }
+    my ($head) = @_;
+    return  if( ! defined $head );
+    printf "%s", $head;
 }
 
 #------------------------------------------------------------------------------
 #------------------------------------------------------------------------------
 sub _printMenueFooter($) {
-     my ($opts) = @_;
-     foreach my $op ( @{$opts} ){
-         next     if( ! $op->{'footer'} );
-         printf "%s\n", $op->{'footer'};
-     }
+    my ($foot) = @_;
+    return  if( ! defined $foot );
+    printf "%s", $foot;
 }
 
 #------------------------------------------------------------------------------
@@ -865,6 +864,25 @@ sub _getRealIndex($$) {
     return $real;
 }
 
+#------------------------------------------------------------------------------
+#------------------------------------------------------------------------------
+sub _normMenueOpts($){
+    my ($opts) = @_;
+
+    my $head = '';
+    my $foot = '';
+    my @OPTS;
+    foreach my $l ( @{$opts} ){
+        $head .= sprintf "%s\n", $l->{'header'}   if( defined $l->{'header'} );
+        $foot .= sprintf "%s\n", $l->{'footer'}   if( defined $l->{'footer'} );
+        push @OPTS, $l                          if( defined $l->{'label'}  );
+    }
+    $head = $head eq '' ? undef : $head;
+    $foot = $foot eq '' ? undef : $foot;
+
+    return \@OPTS, $head, $foot;
+}
+
 #------------------------------------------------------------------------------
 # Display a menue, return the selected index number and the menue data structure.
 # If a VALUE or DEFAULT key of a menue option points to a value this value can
@@ -880,13 +898,14 @@ sub _getRealIndex($$) {
 #------------------------------------------------------------------------------
 sub Menue($)
 {
-    my ($opts) = @_;
+    my ($OPTS) = @_;
 
+    my ($opts,$head,$foot) = _normMenueOpts($OPTS);
     my ($i,$o) = (0,0);
     my $maxLen = _maxLabelLength($opts);
     my $form1 = "%3d %-${maxLen}s ";
     system("clear");
-    _printMenueHeader($opts);
+    _printMenueHeader($head);
     ($i,$o) = (0,0);
     foreach my $op ( @{$opts} )
     {
@@ -898,18 +917,13 @@ sub Menue($)
             printf $form, $i++,$op->{'label'};
         }
     }
-    _printMenueFooter($opts);
+    _printMenueFooter($foot);
     printf "\nSelect: ";
     $o = _getNumber( $i-1);
-    if( $o < $i && $o > -1 )
-    {
-        my $oo = _getRealIndex($o, $opts);
-        _setValue($oo, $opts);
-        _jump($oo, $opts); # jump to callback if defined
-    }
+    _setValue($o, $opts);
+    _jump($o, $opts); # jump to callback if defined
     return $o,$opts;
 }
- 
 #------------------------------------------------------------------------------
 # Prepare input for Data Menue. Allowed input formats:
 # INPUT2: [{'label'=>'Max','value'=>'foo'},{'label'=>'Tim','value'=>99}]
@@ -1083,7 +1097,7 @@ sub _jump($$)
     my $call = $menue->[$o]->{'jump'};
     my $args = defined $menue->[$o]->{'argv'} ? $menue->[$o]->{'argv'} : undef;
 
-    $call->($args);
+    $call->(@{$args});
     return;
 }
 
@@ -1116,41 +1130,98 @@ sub _getDefForm($$)
 
     return $def,"$form\n";
 }
+
+#------------------------------------------------------------------------------
+# Remove last entered character from input. Move cursor one position backward.
+#------------------------------------------------------------------------------
+sub _delLastChar(){
+    printf "%c %c", 0x08, 0x08;     # backspace,blank,backspace
+}
+
+#------------------------------------------------------------------------------
+# Print one character to STDOUT without buffering.
+#------------------------------------------------------------------------------
+sub _flushChar($){
+    my ($char) = @_;
+    my $old = $|;
+    $| = 1;
+    printf "%s", $char;
+    $| = $old;
+    return;
+}
+
+#------------------------------------------------------------------------------
+# Check if first digit exeeds the range if used on ten-spot position. In this
+# case we got a valid number in the range of 0-9.
+#------------------------------------------------------------------------------
+sub _unique($$){
+    my ($o,$maxNum) = @_;
+    if( $$o =~ /^\n$/ ){ # default 0 if got ENTER
+        $$o = 0;
+        return 1;
+    }
+    return 1    if( $$o*10 > $maxNum);
+}
+
+#------------------------------------------------------------------------------
+# Check if first digit is in range. \n is valid for special handling later.
+#------------------------------------------------------------------------------
+sub _invalid($$){
+    my ($o,$maxNum) = @_;
+    return 0    if( $o =~ /^\n$/ );
+    return 0    if( $o <= $maxNum);
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # Read the next number from STDIN. Return 0 if given character is not a digit.
 # Read two characters if max option number is greater than 9. 
 # Valid option numbers are 0...99.
 #------------------------------------------------------------------------------
-sub _getNumber($)
-{
+sub _getNumber($) {
     my ($maxNum) = @_;
 
-    my $o=_getChar();
-    if( $maxNum > 9 )
-    {
-        my $oo=_getChar();
-        $o=10*$o+$oo if( $oo =~ /^\d$/ );
+    my $o;
+    while( 1 ) {
+        $o=_getChar('^\d+$');
+        next   if( _invalid($o,$maxNum) );
+        last   if( _unique(\$o,$maxNum) );
+
+        if( $maxNum > 9 ) {
+            _flushChar($o);
+            my $oo=_getChar('^\d+$');
+            $o=10*$o+$oo;
+        }
+        last   if( $o <= $maxNum && $o > -1 );
+        _delLastChar();
     }
-    return 0 if( $o !~ /^\d+$/ );
+    _delLastChar();
+    _flushChar($o);
     return $o;
 }
 #------------------------------------------------------------------------------
 # Read one character from STDIN. FIXME: stty method is not portable
 #------------------------------------------------------------------------------
-sub _getChar()
+sub _getChar($)
 {
-    system "stty", '-icanon', 'eol', "\001";
-    my $key = getc(STDIN);
-    system "stty", 'icanon', 'eol', '^@'; # ASCII null
-    return $key;
+    my ($patt) = @_;
+
+    while( 1 ) {
+        system "stty", '-echo', '-icanon', 'eol', "\001";
+        my $key = getc(STDIN);
+        system "stty",  'echo', 'icanon', 'eol', '^@'; # ASCII null
+        return $key if( $key =~ /$patt/ );
+        return $key if( $key =~ /\n/ ); # use default
+    }
 }
 
 #------------------------------------------------------------------------------
 # Read a line from STDIN and assign it to the "value" key of an menue option.
-# Data structure: [{label=>,value=>,jump=>,argv=>},...]
+# Data structure: [{label=>,value=>,readOnly=>,jump=>,argv=>},...]
 # After this function value=> has one of the following values:
 # - the read line if not empty
 # - the old value if read an empty line and an old value exists
+# - the value is not changeable if the key readOnly is defined
 #------------------------------------------------------------------------------
 sub _setValue($)
 {
@@ -1162,6 +1233,8 @@ sub _setValue($)
     my $def = defined $op->{'value'} ? $op->{'value'} : '';
 
     printf "\n%s [%s]:", $op->{'label'}, $def;
+    return $def if( defined $op->{'readOnly'} );
+
     my $resp = <STDIN>;
     chomp $resp;
 
